var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
import { syntaxTree } from "@codemirror/language";
import { StateField } from "@codemirror/state";
import {
  Decoration,
  WidgetType,
  EditorView
} from "@codemirror/view";
import { image as classes } from "../classes";
const imageURLStateField = StateField.define({
  create(state) {
    return extractImages(state);
  },
  update(value, tx) {
    if (tx.docChanged)
      return extractImages(tx.state);
    return value.map(tx.changes);
  },
  provide(field) {
    return EditorView.decorations.from(field);
  }
});
function extractImages(state) {
  const imageUrls = [];
  syntaxTree(state).iterate({
    enter: ({ name, node, from, to }) => {
      if (name !== "Image")
        return;
      const alt = state.sliceDoc(from, to).match(/(?:!\[)(.*?)(?:\])/).pop();
      const urlNode = node.getChild("URL");
      if (urlNode) {
        const url = state.sliceDoc(urlNode.from, urlNode.to);
        imageUrls.push({ src: url, from, to, alt });
      }
    }
  });
  const widgets = imageUrls.map((info) => Decoration.widget({
    block: true,
    widget: new ImagePreviewWidget(info),
    side: 1
  }).range(info.to));
  return Decoration.set(widgets, true);
}
class ImagePreviewWidget extends WidgetType {
  constructor(imageInfo) {
    super();
    this.imageInfo = imageInfo;
    __publicField(this, "preloaded");
    this.preloaded = preloadImage(this.imageInfo.src);
  }
  get estimatedHeight() {
    return this.preloaded.height || -1;
  }
  toDOM(view) {
    const img = this.preloaded.cloneNode();
    img.setAttribute("draggable", "false");
    img.classList.add(classes.widget);
    img.alt = this.imageInfo.alt;
    if (img.height > 0)
      img.style.height = String(img.height) + "px";
    img.onload = img.onerror = () => {
      view.requestMeasure();
    };
    return img;
  }
  eq(widget) {
    return JSON.stringify(widget.imageInfo) === JSON.stringify(this.imageInfo);
  }
}
function preloadImage(url) {
  const img = new Image();
  img.src = url;
  return img;
}
export {
  imageURLStateField
};
//# sourceMappingURL=image.js.map
